Package: correspondenceTables
Type: Package
Title: Creating Correspondence Tables Between Two Statistical
        Classifications
Date: 2026-05-15
Version: 1.0.2
Authors@R: c(person("Vasilis", "Chasiotis", role = "aut", comment="Department of Statistics, Athens University of Economics and Business"), 
      person("Photis", "Stavropoulos", role = "aut", comment="Quantos S.A. Statistics and Information Systems"),
      person("Martin", "Karlberg", role = "aut"),
      person("Mátyás", "Mészáros", email = "matyas.meszaros@ec.europa.eu", role = "cre"),
      person("Martina", "Patone", role = "aut"),
      person("Erkand", "Muraku", role = "aut"), 
      person("Clement", "Thomas", role = "aut"), 
      person("Loic", "Bienvenu", role = "aut"),
      person("Mauro", "Baldacchini", role = "aut"),
      person("Khadija", "Sossey-Lallemand", role = "aut"),
      person("Mark", "van der Loo", role = "aut"))
Description: 
    A candidate correspondence table between two classifications can be created when there are correspondence tables leading from the first classification to the second one via intermediate 'pivot' classifications. 
    The correspondence table between two statistical classifications can be updated when one of the classifications gets updated to a new version.
License: EUPL
Encoding: UTF-8
Imports: data.table, httr, stringr, igraph
Suggests: knitr, rmarkdown, tinytest
VignetteBuilder: knitr
NeedsCompilation: no
URL: https://github.com/eurostat/correspondenceTables
BugReports: https://github.com/eurostat/correspondenceTables/issues
Maintainer: Mátyás Mészáros <matyas.meszaros@ec.europa.eu>
RoxygenNote: 7.3.3
Packaged: 2026-05-15 13:15:57 UTC; rstudio
Author: Vasilis Chasiotis [aut] (Department of Statistics, Athens University of
    Economics and Business),
  Photis Stavropoulos [aut] (Quantos S.A. Statistics and Information
    Systems),
  Martin Karlberg [aut],
  Mátyás Mészáros [cre],
  Martina Patone [aut],
  Erkand Muraku [aut],
  Clement Thomas [aut],
  Loic Bienvenu [aut],
  Mauro Baldacchini [aut],
  Khadija Sossey-Lallemand [aut],
  Mark van der Loo [aut]
Repository: CRAN
Date/Publication: 2026-05-15 15:50:15 UTC
Built: R 4.6.0; ; 2026-05-15 18:28:33 UTC; unix
