# Generated by roxygen2: do not edit by hand

export(aggregateCorrespondenceTable)
export(analyseCorrespondenceTable)
export(classificationList)
export(classificationQC)
export(correspondenceTableList)
export(dataStructure)
export(newCorrespondenceTable)
export(prefixList)
export(retrieveClassificationTable)
export(retrieveCorrespondenceTable)
export(updateCorrespondenceTable)
import(httr)
import(igraph)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(httr,GET)
importFrom(httr,RETRY)
importFrom(httr,accept)
importFrom(httr,add_headers)
importFrom(httr,content)
importFrom(httr,stop_for_status)
importFrom(httr,timeout)
importFrom(httr,user_agent)
importFrom(igraph,decompose.graph)
importFrom(igraph,graph.data.frame)
importFrom(stats,aggregate)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,menu)
importFrom(utils,read.csv)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
importFrom(utils,write.csv)
