% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prefixList.R
\name{prefix_declared_in_block}
\alias{prefix_declared_in_block}
\title{Validate that a prefix token is declared in a prefix block (pure)}
\usage{
prefix_declared_in_block(token, prefix_block)
}
\arguments{
\item{token}{a candidate prefix token (will be normalized)}

\item{prefix_block}{a single string containing the PREFIX lines}
}
\value{
TRUE if declared, FALSE otherwise
}
\description{
Validate that a prefix token is declared in a prefix block (pure)
}
\keyword{internal}
