% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prefixList.R
\name{normalize_namespace_iri}
\alias{normalize_namespace_iri}
\title{Normalize a namespace IRI (pure)}
\usage{
normalize_namespace_iri(uri)
}
\arguments{
\item{uri}{character vector}
}
\value{
normalized IRIs
}
\description{
Ensures each IRI:
- has an HTTP(S) scheme (defaults to https if missing),
- has no whitespace,
- ends with exactly one trailing slash.
}
\keyword{internal}
